--------------------
--01 Static Entity--
--------------------

function newStaticEntity(image, x, y)
  local entity = {
    x  = x or 0, -- Coordinates
    y  = y or 0, 
    r  = 0,      -- Rotation
    sx = 1,      -- Scale
    sy = 1, 
  }

  function entity.draw()
    love.graphics.draw(image, entity.x, entity.y)
  end

  function entity.setPosition(x, y)
    entity.x = x or entity.x
    entity.y = y or entity.y
  end
  
  function entity.move(dx, dy)
    entity.x = entity.x + (dx or 0)
    entity.y = entity.y + (dy or 0)
  end
  
  return entity
end

----------------------
--01 Animated Entity--
----------------------

function newAnimatedEntity(image, x, y, width, height)
  local entity = newStaticEntity(image, x, y) -- Yeah, right. Too lazy to type that again.
   
  -- The Basic Animation Things
  local animations = {}
  local current_animation = nil
  local frame_timer = 0
  local current_frame = 1
  local looping = true
  entity.animation_playing = true
  
  function entity.addAnimation(id, frame_rate)
    animations[id] = {frames = {}, frame_rate = frame_rate}
  end
  
  function entity.addFrames(id, x, y, intervals)
    for i = 0, intervals - 1 do
      local start_x = x + ((width+1) * i)
      local quad = love.graphics.newQuad(start_x, y, width, height, image:getWidth(), image:getHeight())
      table.insert(animations[id].frames, quad)
    end
  end
   
  function entity.setAnimation(id, loop)
    -- Reset these values if switching to ANOTHER animation
    if current_animation ~= id then
      current_animation = id
      frame_timer = 0
      current_frame = 1
      looping = loop == nil and true or loop
      entity.animation_playing = true
    end
  end
   
  function entity.updateAnimation(dt)
    -- Detect first if currently set animation has more than 1 frame in it
    local max_frames = #animations[current_animation].frames
    if max_frames > 1 and entity.animation_playing == true then
      frame_timer = frame_timer + dt

      -- Pause Animation if not set to Looping
      if looping == false and current_frame >= max_frames then entity.animation_playing = false end

      -- Move to next frame according to frame rate
      local frame_rate = animations[current_animation].frame_rate
      if frame_timer >= frame_rate then
        frame_timer = frame_timer - frame_rate
        current_frame = current_frame >= max_frames and 1 or current_frame + 1
      end

    end
 end
   
  function entity.draw()
    love.graphics.draw(image, animations[current_animation].frames[current_frame], entity.x, entity.y)
  end
   
  return entity
end
