local x = 0
local y = 0
local scale = 1

screen = {
  width = love.graphics.getWidth(),
  height = love.graphics.getHeight(),
}

-- Used in love.resize(w, h)
function screen.resize(w, h)
  local sw = w/screen.width
  local sh = h/screen.height
  
  -- If the height doesn't fit, scale to height. Otherwise, scale to width
  scale = h < screen.height * sw and sh or sw
  
  -- Centering the Coordinates
  x = w/scale/2 - screen.width/2
  y = h/scale/2 - screen.height/2
  
  --Disabling any drawing to be drawn outside the screen.
  love.graphics.setScissor(x * scale, y * scale, screen.width * scale, screen.height * scale)
end

-- Load Window Settings (width, height, fullscreen)
function screen.loadConfig()
  if love.filesystem.getInfo('screen.cfg') then
    local data = {}
    for line in love.filesystem.lines('screen.cfg') do
      table.insert(data, line)
    end
    love.window.setMode(tonumber(data[1]), tonumber(data[2]), {fullscreen = data[3] == 'true'})
  end
end

function screen.saveConfig(width, height, fullscreen)
  love.filesystem.write('screen.cfg',
    width .. '\n' ..
    height .. '\n' ..
    (fullscreen and 'true' or 'false')
  )
end

-- Forces resolution to w and h no matter the window size
function screen.set(w, h)
  screen.width = w
  screen.height = h
  screen.resize(love.graphics.getWidth(), love.graphics.getHeight())
end

function screen.getMouseX()
  return love.mouse.getX() / scale - x
end
function screen.getMouseY()
  return love.mouse.getY() / scale - y
end
function screen.getMousePosition()
  return screen.getMouseX(), screen.getMouseY()
end

function screen.convertX(mx)
  return mx / scale - x
end
function screen.convertY(my)
  return my / scale - y
end

function screen.convertCoordinates(mx, my)
   return screen.convertX(mx), screen.convertY(my)
end

function screen.draw()
  love.graphics.scale(scale)
  love.graphics.translate(x, y)
end
