local default_color = {1, 1, 1}
local default_font = love.graphics.newFont()

state_machine = { states = {} }

-- State Template
function newState(backgroundcolor, default_color)
  local state_template = {}
  function state_template.load(vars)                  end
  function state_template.update(dt)                  end
  function state_template.draw()                      end
  function state_template.keypressed(key)             end
  function state_template.keyreleased(key)            end
  function state_template.textinput(t)                end
  function state_template.mousemoved(x, y, dx, dy)    end
  function state_template.mousepressed(x, y, button)  end
  function state_template.mousereleased(x, y, button) end
  function state_template.wheelmoved(x, y)            end
   
  return state_template
end

-- STACK-TYPE STATE MACHINE
function state_machine.push(state, vars)
  table.insert(state_machine.states, state)
  state_machine.states[#state_machine.states].load(vars)
end
function state_machine.pop()
  table.remove(state_machine.states, #state_machine.states)
end
function state_machine.change(state, vars)
  state_machine.states = {}
  state_machine.push(state, vars)
end

function state_machine.update(dt)
  if #state_machine.states > 0 then
    state_machine.states[#state_machine.states].update(dt)
  end
end
-- Draw everything in the stack (Useful for Pauses)
function state_machine.draw()
  if #state_machine.states > 0 then
    for i = 1, #state_machine.states do
      love.graphics.setColor(default_color)
      love.graphics.setFont(default_font)
      state_machine.states[i].draw()
    end
  end
end

-- Keyboard Inputs
function state_machine.keypressed(key)
  if #state_machine.states > 0 then
    state_machine.states[#state_machine.states].keypressed(key)
  end
end
function state_machine.keyreleased(key)
  if #state_machine.states > 0 then
    state_machine.states[#state_machine.states].keyreleased(key)
  end
end
function state_machine.textinput(t)
  if #state_machine.states > 0 then
    state_machine.states[#state_machine.states].textinput(t)
  end
end

-- Mouse Functions
function state_machine.mousemoved(x, y, dx, dy)
  if #state_machine.states > 0 then
    state_machine.states[#state_machine.states].mousemoved(x, y, dx, dy)
  end
end
function state_machine.mousepressed(x, y, button)
  if #state_machine.states > 0 then
    state_machine.states[#state_machine.states].mousepressed(x, y, button)
  end
end
function state_machine.mousereleased(x, y, button)
  if #state_machine.states > 0 then
    state_machine.states[#state_machine.states].mousereleased(x, y, button)
  end
end
function state_machine.wheelmoved(x, y)
  if #state_machine.states > 0 then
    state_machine.states[#state_machine.states].wheelmoved(x, y)
  end
end


