-- Preferrably uses Monospace Fonts
local blinkrate = 0.5

function newTextInput(font, x, y, maxlength, vars)
  local blinktimer = 0
  local blinkchar = '_'
  local minwrap = (font:getWidth('W') * maxlength) / 2 -- I think "W" is the widest character, but it's dynamic
  local wrap = minwrap

  local padding = vars and vars.padding     or 2
  local txcolor = vars and vars.textcolor   or {1,1,1}
  local bxcolor = vars and vars.boxcolor    or {0,0,0,0.5}
  local bdcolor = vars and vars.bordercolor or {1,1,1}
  local rx      = vars and vars.rx          or 0
  local ry      = vars and vars.ry          or 0

  local textbox = {
    x = x, y = y,
    text = text or '',
    active = true
  }
  
  function textbox.update(dt)
    if textbox.active then
      if textbox.text:len() < maxlength then
        blinktimer = blinktimer + dt
        if blinktimer >= blinkrate then
          blinktimer = 0
        
          -- Inline Condition: var = <cond> and <var_true> or <var_false>
          blinkchar = blinkchar == '_' and ' ' or '_'
        end
      else
        -- Don't blink when maxlength is reached
        blinkchar = ''
      end
    end
  end
  
  function textbox.draw()
    if textbox.active then
      local h = font:getHeight('W') + (padding * 2)
      local w = wrap + (padding * 2)

      -- Filling the box
      love.graphics.setColor(bxcolor)
      love.graphics.rectangle('fill', x - padding, y - padding, w, h, rx, ry)    

      -- Same thing for the border, except 'line'
      love.graphics.setColor(bdcolor)
      love.graphics.rectangle('line', x - padding, y - padding, w, h, rx, ry)
      
      love.graphics.setFont(font)
      love.graphics.print({txcolor, textbox.text .. blinkchar}, x, y)
    end
  end
  
  function textbox.textinput(t)                           -- I assume that (\ ' ") will produce bugs
    if textbox.active and textbox.text:len() < maxlength and not (t == '\\' or t == "'" or t == '"') then
      textbox.text = textbox.text .. t
      
      -- Re-adjust display to textlength
      local textwrap = textbox.text:len() == maxlength and font:getWidth(textbox.text) or font:getWidth(textbox.text .. '_')
      wrap = textwrap > minwrap and textwrap or minwrap
    end
  end
  
  function textbox.keypressed(key)
    if textbox.active and key == 'backspace' then
      textbox.text = textbox.text:sub(0, textbox.text:len() - 1)
      local textwrap = font:getWidth(textbox.text .. '_')
      wrap = minwrap > textwrap and minwrap or textwrap
    end
  end
  
  return textbox
end
