require 'lib/screen'
require 'lib/statemachine'
require 'lib/entities'
require 'lib/menu'

require 'src/objects/board'

require 'src/states/title'
require 'src/states/tutorial'
require 'src/states/selectboard'
require 'src/states/selectplayers'
require 'src/states/play'
require 'src/states/moving'
require 'src/states/pause'
require 'src/states/gameover'

function love.load()
  -- Don't blur images when zoomed in/out
  love.graphics.setDefaultFilter('nearest', 'nearest')
  
  g_image = love.graphics.newImage('img/board.png')
  g_tilewidth  = 32
  g_tileheight = 30
  g_font = love.graphics.newFont('font/round pop.ttf', 22, 'mono')
  
  -- Force Resolution No Matter the Window Size
  screen.set(320,224) -- Sega Genesis Resolution

  -- My main.lua is heavily tied to state_machine
  state_machine.push(state_title, true)
end

function love.update(dt)
  board.update(dt)
  state_machine.update(dt)
end
function love.draw()
  -- Forced Resolution Draw
  screen.draw()

  -- Put universal draws here
  love.graphics.setColor(1,1,1)
  board.draw()
  -- State Machine
  state_machine.draw()
end

-- Keyboard Inputs
function love.keypressed(key)
  state_machine.keypressed(key)
end
function love.keyreleased(key)
  state_machine.keyreleased(key)
end
function love.textinput(t)
  state_machine.textinput(t)
end

-- Mouse Functions
function love.mousemoved(x, y, dx, dy)
  state_machine.mousemoved(screen.convertX(x), screen.convertY(y), dx, dy)
end
function love.mousepressed(x, y, button)
  state_machine.mousepressed(screen.convertX(x), screen.convertY(y), button)
end
function love.mousereleased(x, y, button)
  state_machine.mousereleased(screen.convertX(x), screen.convertY(y), button)
end
function love.wheelmoved(x, y)
  state_machine.wheelmoved(x, y)
end

-- Window Functions
function love.resize(w, h)
  screen.resize(w, h)
end
