local Tiles = require 'lib/tilemanager'
local Pieces = require 'src/objects/pieces'

local notations = {
  void     = ' ',
  goal     = 'G',
  rink     = 'R',
  slider   = 'S'
}
board = {}

local files, ranks, squares, move_history, history_position, initial_rep -- initial_rep is for reloading the same board

--GUI
local boardx, boardy, boardwidth, boardheight, boardscale
local rink, move_quads, gui_moves
local speed = 500

function board.updateRink()
  local new_rep = {}
  
  for r = 1, ranks do
    local rank = ''
    for c = 1, files do
      local s = ((r-1) * files) + c
      
      if s == board.goal then
        rank = rank .. notations.goal
      elseif squares[s] == notations.void then
        rank = rank .. notations.void
      else
        rank = rank .. notations.rink
      end
    end
    table.insert(new_rep, rank)
  end

  rink.loadMap(new_rep)
  rink.update()
end

function board.getFile(index)
  return ((index - 1) % files) + 1
end
function board.getRank(index)
  return math.ceil(index / files)
end
function board.getIndex(file, rank)
  return ((rank - 1) * files) + file
end

function board.load(f, r, representation, g)
  files = f
  ranks = r
  initial_rep = representation

  squares       = {}
  board.sliders = {}
  board.players = {}
  move_history  = {}
  history_position = #move_history
  
  board.goal = g
  board.turn_counter = 1

  -- Looking at each character in the Inputted Representation
  for i = 1, representation:len() do
    -- Putting each character in the Squares table
    local c = representation:sub(i,i)
    table.insert(squares, c)
    
    if board.goal == nil and c == notations.goal then board.goal = i end
    
    -- If it's a slider
    if c == notations.slider then
      table.insert(board.sliders, newSlider(i))
    
    -- If it's not a slider, see if it's a player.
    else
      local f = 0
      for k, v in pairs(notations) do
        if v == c then
          f = 1
        end
      end
      if f == 0 then
        board.players[tonumber(c)] = newPlayer(c, i)
        table.insert(board.sliders, newSlider(i))
      end
    end
  end

  --[[GUI --]]
  rink = newTileSet(g_image, g_tilewidth, g_tileheight, files * ranks)
  rink.setDictionary({'GR'})
  board.updateRink()

  gui_moves = {}
  move_quads = 
  { love.graphics.newQuad(  0, 93, g_tilewidth, g_tileheight, g_image:getWidth(), g_image:getHeight()),
    love.graphics.newQuad( 33, 93, g_tilewidth, g_tileheight, g_image:getWidth(), g_image:getHeight())
  }
  board.draw_moves = false

  -- Scaling and Translating
  boardwidth = screen.width / (g_tilewidth * files)
  boardheight = screen.height / (g_tileheight * ranks)
  boardscale = boardheight < screen.height * boardwidth and boardheight or boardwidth
  boardx = screen.width /boardscale/2 - (g_tilewidth  * files)/2
  boardy = screen.height/boardscale/2 - (g_tileheight * ranks)/2
end

function board.update(dt)
  for _, v in pairs(board.sliders) do v.updateAnimation(dt) end
  for _, v in pairs(board.players) do v.updateAnimation(dt) end

  if board.draw_moves then
    local move = gui_moves['up']
    if move.index > 0 and move.y > (board.getRank(move.index) - 1) * g_tileheight then
      gui_moves['up'].y = move.y - speed * dt
      if gui_moves['up'].y <= (board.getRank(move.index) - 1) * g_tileheight then
        gui_moves['up'].y = (board.getRank(move.index) - 1) * g_tileheight
      end
    end
    
    move = gui_moves['down']
    if move.index > 0 and move.y < (board.getRank(move.index) - 1) * g_tileheight then
      gui_moves['down'].y = move.y + speed * dt
      if gui_moves['down'].y >= (board.getRank(move.index) - 1) * g_tileheight then
        gui_moves['down'].y = (board.getRank(move.index) - 1) * g_tileheight
      end
    end

    move = gui_moves['right']
    if move.index > 0 and move.x < (board.getFile(move.index) - 1) * g_tilewidth then
      gui_moves['right'].x = move.x + speed * dt
      if gui_moves['right'].x >= (board.getFile(move.index) - 1) * g_tilewidth then
        gui_moves['right'].x = (board.getFile(move.index) - 1) * g_tilewidth
      end
    end

    move = gui_moves['left']
    if move.index > 0 and move.x > (board.getFile(move.index) - 1) * g_tilewidth then
      gui_moves['left'].x = move.x - speed * dt
      if gui_moves['left'].x <= (board.getFile(move.index) - 1) * g_tilewidth then
        gui_moves['left'].x = (board.getFile(move.index) - 1) * g_tilewidth
      end
    end
  end

end

function board.draw()
  love.graphics.push()
  love.graphics.scale(boardscale)
  love.graphics.translate(boardx, 0)
  
  rink.draw()
  for _, v in pairs(board.sliders) do v.draw() end
  for _, v in pairs(board.players) do v.draw() end

  if board.draw_moves then
    for k, v in pairs(gui_moves) do
      if v.index > 0 then
        love.graphics.draw(g_image, move_quads[v.slide and 1 or 2], v.x, v.y)
      end
    end
  end
  love.graphics.pop()
end

function board.checkMoves(i)
  local t
  board.players[i].moves = {
    ['up']    = {index = 0, slide = false},
    ['down']  = {index = 0, slide = false},
    ['left']  = {index = 0, slide = false},
    ['right'] = {index = 0, slide = false}
  }

  -- Up
  t = board.players[i].index-files
  if squares[t] == notations.slider then
    board.players[i].moves['up'].index = t
  elseif (squares[t] == notations.rink or squares[t] == notations.goal) and board.getRank(t) > 0 then
    while board.getRank(t) > 0 and (squares[t] == notations.rink or squares[t] == notations.goal) do
      t = t-files
    end

    board.players[i].moves['up'].index = t+files
    board.players[i].moves['up'].slide = true
  end

  -- Down
  t = board.players[i].index+files
  if squares[t] == notations.slider then
    board.players[i].moves['down'].index = t
  elseif (squares[t] == notations.rink or squares[t] == notations.goal) and board.getRank(t) <= ranks then
    while board.getRank(t) <= ranks and (squares[t] == notations.rink or squares[t] == notations.goal) do
      t = t+files
    end

    board.players[i].moves['down'].index = t-files
    board.players[i].moves['down'].slide = true
  end

  -- Right
  t = board.players[i].index
  if board.getFile(t) < files and squares[t+1] == notations.slider then
    board.players[i].moves['right'].index = t+1
    table.insert(board.players[i].moves['right'], { index = t+1, slide = false})
  elseif board.getFile(t) < files and (squares[t+1] == notations.rink or squares[t+1] == notations.goal) then
    repeat
      t = t+1
    until board.getFile(t) == files or not (squares[t+1] == notations.rink or squares[t+1] == notations.goal)
    board.players[i].moves['right'].index = t
    board.players[i].moves['right'].slide = true
  end

  -- Left
  t = board.players[i].index
  if board.getFile(t) > 1 and squares[t-1] == notations.slider then
    board.players[i].moves['left'].index = t-1

  elseif board.getFile(t) > 1 and (squares[t-1] == notations.rink or squares[t-1] == notations.goal) then
    repeat
      t = t-1
    until board.getFile(t) == 1 or not (squares[t-1] == notations.rink or squares[t-1] == notations.goal)
    board.players[i].moves['left'].index = t
    board.players[i].moves['left'].slide = true
  end
end

-- 3. Move
function board.move(from, to, slide, redo)
  if from ~= to then
    squares[to] = squares[from]
    squares[from] = slide and (from == goal and notations.goal or notations.rink) or notations.slider
    
    if slide then
      for _, v in pairs(board.sliders) do
        if v.index == from then
          v.index = to
        end
      end
    end
    
    for _, v in pairs(board.players) do
      if v.index == from then
        v.index = to
      end
    end
  end
end

-- 4. Undo/Redo/Goto Recorded Move
function board.undo()
  if board.history_position > 0 then
  end
end

function board.redo()
  if board.history_position < #board.move_history then
  end
end

function board.gotoMove(index)
  if board.history_position < index then
    repeat
      board.redo()
    until board.history_position == index
  elseif board.history_position > index then
    repeat
      board.undo()
    until board.history_position == index
  end
end

-- 5. Evaluation
function board.evaluate()
  local evaluation = ''
  for i, v in pairs(board.players) do
    if v.index == goal then
      evaluation = i
    end
  end
  return evaluation
end

function board.reset()
  local ai = {}
  for i = 1, #board.players do table.insert(ai, board.players[i].ai) end
  board.load(files, ranks, initial_rep)
  for i = 1, #board.players do board.players[i].ai = ai[i] end
end

-- GUI ELEMENT FUNCTIONS

function board.updateGuiMoves()
  gui_moves = board.players[board.turn_counter].moves

  for _, v in pairs(board.sliders) do
    v.setAnimation(v.index == board.players[board.turn_counter].index and 'active' or 'idle')
  end

  for _, v in pairs(gui_moves) do
    v.x = (board.getFile(board.players[board.turn_counter].index) - 1) * g_tilewidth
    v.y = (board.getRank(board.players[board.turn_counter].index) - 1) * g_tileheight
  end
end
