state_moving = newState()

local slider_index, from, to, slide, speed, direction
local slide_sound = love.audio.newSource('sfx/slide.flac', 'static')
local bump_sound = love.audio.newSource('sfx/bump.flac', 'static')

function state_moving.load(vars)
  from = vars.from
  to = vars.to
  slide = vars.slide
  speed = slide and 160 or 100
  direction = vars.direction

  board.players[board.turn_counter].setAnimation(slide and 'idle' or 'running')

  for i, v in pairs(board.sliders) do
    v.setAnimation('idle')
    if slide and v.index == from then
      slider_index = i
    end
  end

  board.draw_moves = false

  if slide then
    slide_sound:stop()
    slide_sound:play()
  end
end

function state_moving.update(dt)
  if direction == 'up' then
    board.players[board.turn_counter].move(0, -speed * dt)
  elseif direction == 'down' then
    board.players[board.turn_counter].move(0,  speed * dt)
  elseif direction == 'right' then
    board.players[board.turn_counter].move( speed * dt, 0)
  elseif direction == 'left' then
    board.players[board.turn_counter].move(-speed * dt, 0)
  end
  if slide then board.sliders[slider_index].setPosition(board.players[board.turn_counter].x, board.players[board.turn_counter].y) end

  if (direction == 'up'    and board.players[board.turn_counter].y <= (board.getRank(to) - 1) * 30) or
     (direction == 'down'  and board.players[board.turn_counter].y >= (board.getRank(to) - 1) * 30) or
     (direction == 'right' and board.players[board.turn_counter].x >= (board.getFile(to) - 1) * 32) or
     (direction == 'left'  and board.players[board.turn_counter].x <= (board.getFile(to) - 1) * 32) then

       -- Snap Objects into Position into position
       if slide then
         board.sliders[slider_index].setPosition((board.getFile(to)-1) * 32, (board.getRank(to)-1) * 30)
         
         if from ~= to then
           bump_sound:stop()
           bump_sound:play()
         end
       end
       board.players[board.turn_counter].setPosition((board.getFile(to)-1) * 32, (board.getRank(to)-1) * 30)
       board.players[board.turn_counter].setAnimation('idle')

       -- Internal
       board.move(from, to, slide)
       board.turn_counter = slide and (board.turn_counter % #board.players) + 1 or board.turn_counter

       if to == board.goal then
         state_machine.change(state_gameover)
       else
         board.checkMoves(board.turn_counter)

         board.draw_moves = true
         board.updateGuiMoves()
         board.players[board.turn_counter].setAnimation('selecting')

         state_machine.pop()
       end
  end
end

function state_moving.keypressed(key)
  state_machine.push(state_pause)
end
