state_play = newState()

local pass

function state_play.load()
  board.turn_counter = 1
  board.checkMoves(board.turn_counter)

  board.draw_moves = true
  board.updateGuiMoves()
  board.players[board.turn_counter].setAnimation('selecting')
  pass = false
end

function state_play.update(dt)
  local moves = board.players[board.turn_counter].moves
  
  -- If no moves, Pass
  if moves['up'].index == 0 and moves['down'].index == 0 and moves['left'].index == 0 and moves['right'].index == 0 then
    pass = true
  end
  
  -- If player == AI, then Make a move
  if board.players[board.turn_counter].ai then
    if pass then
      pass = false
      local index = board.players[board.turn_counter].index
      state_machine.push(state_moving, {direction = 'up', from = index, to = index, slide = true})
    else
      local directions = {}

      -- Check for valid directions
      if moves['up'].index > 0 then table.insert(directions, 'up') end
      if moves['down'].index > 0 then table.insert(directions, 'down') end
      if moves['left'].index > 0 then table.insert(directions, 'left') end
      if moves['right'].index > 0 then table.insert(directions, 'right') end
      
      -- Choose a random direction
      local direction = love.math.random(1, #directions)
      
      -- Move to that direction
      local from  = board.players[board.turn_counter].index
      local to    = board.players[board.turn_counter].moves[directions[direction]].index
      local slide = board.players[board.turn_counter].moves[directions[direction]].slide    
      state_machine.push(state_moving, {direction = directions[direction], from = from, to = to, slide = slide})
    end
  end
end

function state_play.draw()
  if pass then
    love.graphics.setFont(g_font)
    love.graphics.setColor(200/255, 5/255, 180/255)
    love.graphics.printf('PASS', 0, screen.height / 2 - g_font:getHeight('PASS') / 2, screen.width, 'center')
  end
end

function state_play.keypressed(key)
  if pass then
    local index = board.players[board.turn_counter].index
    pass = false
    state_machine.push(state_moving, {direction = 'up', from = index, to = index, slide = true})
  else
    if key == 'up' or key == 'down' or key == 'left' or key == 'right' then
      if board.players[board.turn_counter].moves[key].index > 0 then
        local from  = board.players[board.turn_counter].index
        local to    = board.players[board.turn_counter].moves[key].index
        local slide = board.players[board.turn_counter].moves[key].slide
        state_machine.push(state_moving, {direction = key, from = from, to = to, slide = slide})
      end
    elseif key == 'escape' then
      state_machine.push(state_pause)
    end
  end
end
